@@ -222,7 +222,8 @@ void QDeclarativeGeoRouteModel::abortRequest()
 QDeclarativeGeoRoute *QDeclarativeGeoRouteModel::get(int index)
 {
     if (index < 0 || index >= routes_.count()) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, INDEX_INVALID).arg(index);
+        qmlInfo(this) << QStringLiteral("Error, invalid index for get():")
+                      << index;
         return 0;
     }
     return routes_.at(index);
@@ -254,12 +255,12 @@ int QDeclarativeGeoRouteModel::rowCount(const QModelIndex &parent) const
 QVariant QDeclarativeGeoRouteModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, MODEL_INDEX_INVALID);
+        qmlInfo(this) << QStringLiteral("Error in indexing route model's data (invalid index).");
         return QVariant();
     }
 
     if (index.row() >= routes_.count()) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, MODEL_INDEX_OUT_OF_RANGE);
+        qmlInfo(this) << QStringLiteral("Fatal error in indexing route model's data (index overflow).");
         return QVariant();
     }
 
@@ -286,7 +287,14 @@ void QDeclarativeGeoRouteModel::setPlugin(QDeclarativeGeoServiceProvider *plugin
         return;
 
     reset(); // reset the model
+
+    if (plugin_)
+        disconnect(plugin_, SIGNAL(localesChanged()), this, SIGNAL(measurementSystemChanged()));
+    if (plugin)
+        connect(plugin, SIGNAL(localesChanged()), this, SIGNAL(measurementSystemChanged()));
+
     plugin_ = plugin;
+
     if (complete_)
         emit pluginChanged();
 
@@ -309,7 +317,8 @@ void QDeclarativeGeoRouteModel::pluginReady()
     QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
     if (!routingManager || serviceProvider->error() != QGeoServiceProvider::NoError) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, PLUGIN_DOESNOT_SUPPORT_ROUTING).arg(serviceProvider->errorString());
+        qmlInfo(this) << QStringLiteral("Error: Plugin does not support routing.\nError message:")
+                      << serviceProvider->errorString();
         return;
     }
     connect(routingManager, SIGNAL(finished(QGeoRouteReply*)),
@@ -407,6 +416,65 @@ bool QDeclarativeGeoRouteModel::autoUpdate() const
     return autoUpdate_;
 }
 
+/*!
+    \qmlproperty Locale::MeasurementSystem QtLocation::RouteModel::measurementSystem
+
+    This property holds the measurement system which will be used when calculating the route. This
+    property is changed when the \l {QtLocation::Plugin::locales}{Plugin::locales} property of
+    \l {QtLocation::RouteModel::plugin}{plugin} changes.
+
+    If setting this property it must be set after the \l {QtLocation::RouteModel::plugin}{plugin}
+    property is set.
+*/
+void QDeclarativeGeoRouteModel::setMeasurementSystem(QLocale::MeasurementSystem ms)
+{
+    if (!plugin_) {
+        qmlInfo(this) << ROUTE_PLUGIN_NOT_SET;
+        return;
+    }
+
+    QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
+    if (!serviceProvider)
+        return;
+
+    QGeoRoutingManager *routingManager = serviceProvider->routingManager();
+    if (!routingManager) {
+        qmlInfo(this) << ROUTE_MGR_NOT_SET;
+        return;
+    }
+
+    if (routingManager->measurementSystem() == ms)
+        return;
+
+    routingManager->setMeasurementSystem(ms);
+    emit measurementSystemChanged();
+}
+
+QLocale::MeasurementSystem QDeclarativeGeoRouteModel::measurementSystem() const
+{
+    if (!plugin_)
+        return QLocale().measurementSystem();
+
+    QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
+    if (!serviceProvider) {
+        if (plugin_->locales().isEmpty())
+            return QLocale().measurementSystem();
+
+        return QLocale(plugin_->locales().first()).measurementSystem();
+    }
+
+    QGeoRoutingManager *routingManager = serviceProvider->routingManager();
+    if (!routingManager) {
+        qmlInfo(this) << ROUTE_MGR_NOT_SET;
+        if (plugin_->locales().isEmpty())
+            return QLocale().measurementSystem();
+
+        return QLocale(plugin_->locales().first()).measurementSystem();
+    }
+
+    return routingManager->measurementSystem();
+}
+
 /*!
     \internal
 */
@@ -509,7 +577,7 @@ void QDeclarativeGeoRouteModel::update()
         return;
 
     if (!plugin_) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, ROUTE_PLUGIN_NOT_SET);
+        qmlInfo(this) << ROUTE_PLUGIN_NOT_SET;
         return;
     }
 
@@ -519,17 +587,17 @@ void QDeclarativeGeoRouteModel::update()
 
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
     if (!routingManager) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, ROUTE_MGR_NOT_SET);
+        qmlInfo(this) << ROUTE_MGR_NOT_SET;
         return;
     }
     if (!routeQuery_) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, ROUTE_QUERY_NOT_SET);
+        qmlInfo(this) << QStringLiteral("Cannot route, valid query not set.");
         return;
     }
     abortRequest(); // Clear previus requests
     QGeoRouteRequest request = routeQuery_->routeRequest();
     if (request.waypoints().count() < 2) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, ROUTE_WAYPOINTS_NOT_SET);
+        qmlInfo(this) << QStringLiteral("Not enough waypoints for routing.");
         return;
     }
 
@@ -560,8 +628,11 @@ void QDeclarativeGeoRouteModel::routingFinished(QGeoRouteReply *reply)
     qDeleteAll(routes_);
     // Convert routes to declarative
     routes_.clear();
-    for (int i = 0; i < reply->routes().size(); ++i)
-        routes_.append(new QDeclarativeGeoRoute(reply->routes().at(i), this));
+    for (int i = 0; i < reply->routes().size(); ++i) {
+        QDeclarativeGeoRoute *route = new QDeclarativeGeoRoute(reply->routes().at(i), this);
+        QQmlEngine::setContextForObject(route, QQmlEngine::contextForObject(this));
+        routes_.append(route);
+    }
     endResetModel();
 
     setErrorString("");
@@ -820,7 +891,7 @@ void QDeclarativeGeoRouteQuery::setExcludedAreas(const QJSValue &value)
         QGeoRectangle r = parseRectangle(value.property(i), &ok);
 
         if (!ok || !r.isValid()) {
-            qmlInfo(this) << "Unsupported waypoint type";
+            qmlInfo(this) << "Unsupported area type";
             return;
         }
 
@@ -883,7 +954,7 @@ void QDeclarativeGeoRouteQuery::removeExcludedArea(const QGeoRectangle &area)
 
     int index = excludedAreas.lastIndexOf(area);
     if (index == -1) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, CANNOT_REMOVE_AREA);
+        qmlInfo(this) << QStringLiteral("Cannot remove nonexistent area.");
         return;
     }
     excludedAreas.removeAt(index);
@@ -923,7 +994,7 @@ void QDeclarativeGeoRouteQuery::clearExcludedAreas()
 void QDeclarativeGeoRouteQuery::addWaypoint(const QGeoCoordinate &waypoint)
 {
     if (!waypoint.isValid()) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, CANNOT_ADD_INVALID_WAYPOINT);
+        qmlInfo(this) << QStringLiteral("Not adding invalid waypoint.");
         return;
     }
 
@@ -952,7 +1023,7 @@ void QDeclarativeGeoRouteQuery::removeWaypoint(const QGeoCoordinate &waypoint)
 
     int index = waypoints.lastIndexOf(waypoint);
     if (index == -1) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, CANNOT_REMOVE_WAYPOINT);
+        qmlInfo(this) << QStringLiteral("Cannot remove nonexistent waypoint.");
         return;
     }
 
