@@ -44,7 +44,9 @@
 #include "qlocationutils_p.h"
 #include "error_messages.h"
 #include "locationvaluetypehelper_p.h"
+#include "qdoublevector2d_p.h"
 
+#include <QtCore/QScopedValueRollback>
 #include <QtQml/QQmlInfo>
 #include <QtQml/QQmlContext>
 #include <QtQml/private/qqmlengine_p.h>
@@ -94,7 +96,7 @@ QT_BEGIN_NAMESPACE
     the Map grows in direct proportion to the number of points in the polyline.
 
     Like the other map objects, MapPolyline is normally drawn without a smooth
-    appearance. Setting the \l {QtQuick2::Item::opacity}{opacity} property will force the object to
+    appearance. Setting the \l {QtQuick::Item::opacity}{opacity} property will force the object to
     be blended, which decreases performance considerably depending on the hardware in use.
 
     \note MapPolylines are implemented using the OpenGL GL_LINES
@@ -178,8 +180,7 @@ struct Vertex
     QVector2D position;
 };
 
-QGeoMapPolylineGeometry::QGeoMapPolylineGeometry(QObject *parent) :
-    QGeoMapItemGeometry(parent)
+QGeoMapPolylineGeometry::QGeoMapPolylineGeometry()
 {
 }
 
@@ -190,10 +191,10 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
                                                  const QList<QGeoCoordinate> &path)
 {
     bool foundValid = false;
-    qreal minX = -1.0;
-    qreal minY = -1.0;
-    qreal maxX = -1.0;
-    qreal maxY = -1.0;
+    double minX = -1.0;
+    double minY = -1.0;
+    double maxX = -1.0;
+    double maxY = -1.0;
 
     if (!sourceDirty_)
         return;
@@ -204,9 +205,9 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
     srcPointTypes_.clear();
     srcPointTypes_.reserve(path.size());
 
-    QPointF origin, lastPoint, lastAddedPoint;
+    QDoubleVector2D origin, lastPoint, lastAddedPoint;
 
-    qreal unwrapBelowX = 0;
+    double unwrapBelowX = 0;
     if (preserveGeometry_)
         unwrapBelowX = map.coordinateToScreenPosition(geoLeftBound_, false).x();
 
@@ -216,7 +217,7 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
         if (!coord.isValid())
             continue;
 
-        QPointF point = map.coordinateToScreenPosition(coord, false);
+        QDoubleVector2D point = map.coordinateToScreenPosition(coord, false);
 
         // We can get NaN if the map isn't set up correctly, or the projection
         // is faulty -- probably best thing to do is abort
@@ -231,7 +232,7 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
             foundValid = true;
             srcOrigin_ = coord;
             origin = point;
-            point = QPointF(0,0);
+            point = QDoubleVector2D(0,0);
 
             minX = point.x();
             maxX = minX;
@@ -241,8 +242,6 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
             srcPoints_ << point.x() << point.y();
             srcPointTypes_ << QPainterPath::MoveToElement;
             lastAddedPoint = point;
-
-
         } else {
             point -= origin;
 
@@ -264,7 +263,7 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
 
     sourceBounds_ = QRectF(QPointF(minX, minY), QPointF(maxX, maxY));
     geoLeftBound_ = map.screenPositionToCoordinate(
-                                    QPointF(minX + origin.x(), minY + origin.y()), false);
+                                    QDoubleVector2D(minX + origin.x(), minY + origin.y()), false);
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -390,7 +389,7 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
     if (!screenDirty_)
         return;
 
-    QPointF origin = map.coordinateToScreenPosition(srcOrigin_, false);
+    QPointF origin = map.coordinateToScreenPosition(srcOrigin_, false).toPointF();
 
     if (!qIsFinite(origin.x()) || !qIsFinite(origin.y())) {
         clear();
@@ -433,7 +432,7 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
     QPolygonF tri;
     const float *vs = ts.vertices();
     for (int i = 0; i < (ts.vertexCount()/2*2); i += 2) {
-        screenVertices_ << Point(vs[i], vs[i + 1]);
+        screenVertices_ << QPointF(vs[i], vs[i + 1]);
 
         if (!qIsFinite(vs[i]) || !qIsFinite(vs[i + 1]))
             break;
@@ -450,9 +449,8 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
     this->translate( -1 * sourceBounds_.topLeft());
 }
 
-QDeclarativePolylineMapItem::QDeclarativePolylineMapItem(QQuickItem *parent) :
-    QDeclarativeGeoMapItemBase(parent),
-    dirtyMaterial_(true)
+QDeclarativePolylineMapItem::QDeclarativePolylineMapItem(QQuickItem *parent)
+:   QDeclarativeGeoMapItemBase(parent), dirtyMaterial_(true), updatingGeometry_(false)
 {
     setFlag(ItemHasContents, true);
     QObject::connect(&line_, SIGNAL(colorChanged(QColor)),
@@ -575,12 +573,12 @@ void QDeclarativePolylineMapItem::removeCoordinate(const QGeoCoordinate &coordin
     int index = path_.lastIndexOf(coordinate);
 
     if (index == -1) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, COORD_NOT_BELONG_TO).arg("PolylineMapItem");
+        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolylineMapItem");
         return;
     }
 
     if (path_.count() < index + 1) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, COORD_NOT_BELONG_TO).arg("PolylineMapItem");
+        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolylineMapItem");
         return;
     }
     path_.removeAt(index);
@@ -609,6 +607,57 @@ QDeclarativeMapLineProperties *QDeclarativePolylineMapItem::line()
     return &line_;
 }
 
+/*!
+    \internal
+*/
+void QDeclarativePolylineMapItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
+{
+    if (updatingGeometry_ || newGeometry.topLeft() == oldGeometry.topLeft()) {
+        QDeclarativeGeoMapItemBase::geometryChanged(newGeometry, oldGeometry);
+        return;
+    }
+
+    QDoubleVector2D newPoint = QDoubleVector2D(x(),y()) + QDoubleVector2D(geometry_.firstPointOffset());
+    QGeoCoordinate newCoordinate = map()->screenPositionToCoordinate(newPoint, false);
+    if (newCoordinate.isValid()) {
+        double firstLongitude = path_.at(0).longitude();
+        double firstLatitude = path_.at(0).latitude();
+        double minMaxLatitude = firstLatitude;
+        // prevent dragging over valid min and max latitudes
+        for (int i = 0; i < path_.count(); ++i) {
+            double newLatitude = path_.at(i).latitude()
+                    + newCoordinate.latitude() - firstLatitude;
+            if (!QLocationUtils::isValidLat(newLatitude)) {
+                if (qAbs(newLatitude) > qAbs(minMaxLatitude)) {
+                    minMaxLatitude = newLatitude;
+                }
+            }
+        }
+        // calculate offset needed to re-position the item within map border
+        double offsetLatitude = minMaxLatitude - QLocationUtils::clipLat(minMaxLatitude);
+        for (int i = 0; i < path_.count(); ++i) {
+            QGeoCoordinate coord = path_.at(i);
+            // handle dateline crossing
+            coord.setLongitude(QLocationUtils::wrapLong(coord.longitude()
+                               + newCoordinate.longitude() - firstLongitude));
+            coord.setLatitude(coord.latitude()
+                              + newCoordinate.latitude() - firstLatitude - offsetLatitude);
+            path_.replace(i, coord);
+        }
+
+        QGeoCoordinate leftBoundCoord = geometry_.geoLeftBound();
+        leftBoundCoord.setLongitude(QLocationUtils::wrapLong(leftBoundCoord.longitude()
+                           + newCoordinate.longitude() - firstLongitude));
+        geometry_.setPreserveGeometry(true, leftBoundCoord);
+        geometry_.markSourceDirty();
+        updateMapItem();
+        emit pathChanged();
+    }
+
+    // Not calling QDeclarativeGeoMapItemBase::geometryChanged() as it will be called from a nested
+    // call to this function.
+}
+
 /*!
     \internal
 */
@@ -648,6 +697,9 @@ void QDeclarativePolylineMapItem::updateMapItem()
     if (!map() || path_.count() == 0)
         return;
 
+    QScopedValueRollback<bool> rollback(updatingGeometry_);
+    updatingGeometry_ = true;
+
     geometry_.updateSourcePoints(*map(), path_);
     geometry_.updateScreenPoints(*map(), line_.width());
 
@@ -685,58 +737,6 @@ bool QDeclarativePolylineMapItem::contains(const QPointF &point) const
     return geometry_.contains(point);
 }
 
-/*!
-    \internal
-*/
-void QDeclarativePolylineMapItem::dragStarted()
-{
-    geometry_.markFullScreenDirty();
-    updateMapItem();
-}
-
-/*!
-    \internal
-*/
-void QDeclarativePolylineMapItem::dragEnded()
-{
-    QPointF newPoint = QPointF(x(),y()) + geometry_.firstPointOffset();
-    QGeoCoordinate newCoordinate = map()->screenPositionToCoordinate(newPoint, false);
-    if (newCoordinate.isValid()) {
-        double firstLongitude = path_.at(0).longitude();
-        double firstLatitude = path_.at(0).latitude();
-        double minMaxLatitude = firstLatitude;
-        // prevent dragging over valid min and max latitudes
-        for (int i = 0; i < path_.count(); ++i) {
-            double newLatitude = path_.at(i).latitude()
-                    + newCoordinate.latitude() - firstLatitude;
-            if (!QLocationUtils::isValidLat(newLatitude)) {
-                if (qAbs(newLatitude) > qAbs(minMaxLatitude)) {
-                    minMaxLatitude = newLatitude;
-                }
-            }
-        }
-        // calculate offset needed to re-position the item within map border
-        double offsetLatitude = minMaxLatitude - QLocationUtils::clipLat(minMaxLatitude);
-        for (int i = 0; i < path_.count(); ++i) {
-            QGeoCoordinate coord = path_.at(i);
-            // handle dateline crossing
-            coord.setLongitude(QLocationUtils::wrapLong(coord.longitude()
-                               + newCoordinate.longitude() - firstLongitude));
-            coord.setLatitude(coord.latitude()
-                              + newCoordinate.latitude() - firstLatitude - offsetLatitude);
-            path_.replace(i, coord);
-        }
-
-        QGeoCoordinate leftBoundCoord = geometry_.geoLeftBound();
-        leftBoundCoord.setLongitude(QLocationUtils::wrapLong(leftBoundCoord.longitude()
-                           + newCoordinate.longitude() - firstLongitude));
-        geometry_.setPreserveGeometry(true, leftBoundCoord);
-        geometry_.markSourceDirty();
-        updateMapItem();
-        emit pathChanged();
-    }
-}
-
 //////////////////////////////////////////////////////////////////////
 
 /*!
