@@ -41,8 +41,8 @@
 
 #include "qdeclarativepositionsource_p.h"
 #include "qdeclarativeposition_p.h"
-#include "error_messages.h"
 
+#include <QtCore/QCoreApplication>
 #include <QtQml/qqmlinfo.h>
 #include <QtQml/qqml.h>
 #include <qnmeapositioninfosource.h>
@@ -289,7 +289,7 @@ void QDeclarativePositionSource::setNmeaSource(const QUrl &nmeaSource)
                 QTimer::singleShot(0, this, SLOT(start()));
             }
         } else {
-            qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, MISSED_NMEA_FILE) << localFileName;
+            qmlInfo(this) << QStringLiteral("Nmea file not found") << localFileName;
 #ifdef QDECLARATIVE_POSITION_DEBUG
             qDebug() << "QDeclarativePositionSource NMEA File was not found: " << localFileName;
 #endif
@@ -370,19 +370,7 @@ void QDeclarativePositionSource::socketError(QAbstractSocket::SocketError error)
 
 void QDeclarativePositionSource::setPosition(const QGeoPositionInfo &pi)
 {
-    if (pi.isValid()) {
-        m_position.setTimestamp(pi.timestamp());
-        m_position.setCoordinate(pi.coordinate());
-        if (pi.hasAttribute(QGeoPositionInfo::GroundSpeed))
-            m_position.setSpeed(pi.attribute(QGeoPositionInfo::GroundSpeed));
-        if (pi.hasAttribute(QGeoPositionInfo::HorizontalAccuracy))
-            m_position.setHorizontalAccuracy(pi.attribute(QGeoPositionInfo::HorizontalAccuracy));
-        if (pi.hasAttribute(QGeoPositionInfo::VerticalAccuracy))
-            m_position.setVerticalAccuracy(pi.attribute(QGeoPositionInfo::VerticalAccuracy));
-    } else {
-        m_position.invalidate();
-    }
-
+    m_position.setPosition(pi);
     emit positionChanged();
 }
 
@@ -464,14 +452,8 @@ int QDeclarativePositionSource::updateInterval() const
 QDeclarativePositionSource::PositioningMethods QDeclarativePositionSource::supportedPositioningMethods() const
 {
     if (m_positionSource) {
-        QGeoPositionInfoSource::PositioningMethods methods = m_positionSource->supportedPositioningMethods();
-        if ( (methods & QGeoPositionInfoSource::AllPositioningMethods) == methods ) {
-            return QDeclarativePositionSource::AllPositioningMethods;
-        } else if (methods & QGeoPositionInfoSource::SatellitePositioningMethods) {
-            return QDeclarativePositionSource::SatellitePositioningMethods;
-        } else if (methods & QGeoPositionInfoSource::NonSatellitePositioningMethods) {
-            return QDeclarativePositionSource::NonSatellitePositioningMethods;
-        }
+        return static_cast<QDeclarativePositionSource::PositioningMethods>(
+            int(m_positionSource->supportedPositioningMethods()));
     }
     return QDeclarativePositionSource::NoPositioningMethods;
 }
@@ -515,16 +497,8 @@ void QDeclarativePositionSource::setPreferredPositioningMethods(PositioningMetho
 QDeclarativePositionSource::PositioningMethods QDeclarativePositionSource::preferredPositioningMethods() const
 {
     if (m_positionSource) {
-        QGeoPositionInfoSource::PositioningMethods methods =
-            m_positionSource->preferredPositioningMethods();
-        if ( (methods & QGeoPositionInfoSource::AllPositioningMethods) == methods) {
-            return QDeclarativePositionSource::AllPositioningMethods;
-        } else if (methods & QGeoPositionInfoSource::SatellitePositioningMethods) {
-            return QDeclarativePositionSource::SatellitePositioningMethods;
-        } else if (methods & QGeoPositionInfoSource::NonSatellitePositioningMethods) {
-            return QDeclarativePositionSource::NonSatellitePositioningMethods;
-        } else if (methods == QGeoPositionInfoSource::NoPositioningMethods)
-            return QDeclarativePositionSource::NoPositioningMethods;
+        return static_cast<QDeclarativePositionSource::PositioningMethods>(
+            int(m_positionSource->preferredPositioningMethods()));
     }
     return m_preferredPositioningMethods;
 }
@@ -662,7 +636,7 @@ void QDeclarativePositionSource::positionUpdateReceived(const QGeoPositionInfo &
 /*!
     \qmlproperty enumeration PositionSource::sourceError
 
-    This property holds the error which last occured with the PositionSource.
+    This property holds the error which last occurred with the PositionSource.
 
     \list
     \li PositionSource.AccessError - The connection setup to the remote positioning backend failed because the
