@@ -308,7 +308,7 @@ QGeoServiceProvider::PlacesFeatures QGeoServiceProvider::placesFeatures() const
  * methods we need to call. Ideally it would be nice to find a way to embed
  * these into the manager() template. */
 template <class Engine>
-Engine *createEngine(QGeoServiceProviderPrivate *d_ptr)
+Engine *createEngine(QGeoServiceProviderPrivate *)
 {
     return 0;
 }
@@ -641,7 +641,7 @@ void QGeoServiceProviderPrivate::loadMeta()
     }
 }
 
-void QGeoServiceProviderPrivate::loadPlugin(const QMap<QString, QVariant> &parameters)
+void QGeoServiceProviderPrivate::loadPlugin(const QVariantMap &parameters)
 {
     Q_UNUSED(parameters)
 
