@@ -45,10 +45,11 @@
 #include "qgeomapscene_p.h"
 #include "qgeocameratiles_p.h"
 #include "qgeocameradata_p.h"
-#include "qgeoprojection_p.h"
-#include "qdoublevector2d_p.h"
 #include "qgeotilecache_p.h"
 
+#include <QtPositioning/private/qgeoprojection_p.h>
+#include <QtPositioning/private/qdoublevector2d_p.h>
+
 #include <Qt3D/qglscenenode.h>
 #include <Qt3D/qgltexture2d.h>
 
@@ -93,7 +94,7 @@ class tst_QGeoMapScene : public QObject
         double mercatorX;
         double mercatorY;
 
-        double halfLength = 1 / (pow(2.0, zoom) * 2);
+        double halfLength = 1 / (std::pow(2.0, zoom) * 2);
         double scaleX = screenWidth / tileSize;
         double scaleY = screenHeight / tileSize;
         double scaledHalfLengthX = halfLength * scaleX;
@@ -345,7 +346,7 @@ class tst_QGeoMapScene : public QObject
             mapGeometry.setCameraData(camera);
             mapGeometry.setVisibleTiles(ct.tiles());
 
-            QPointF point(screenX,screenY);
+            QDoubleVector2D point(screenX,screenY);
             QDoubleVector2D mecartorPos = mapGeometry.screenPositionToMercator(point);
 
             QCOMPARE(mecartorPos.x(),mercatorX);
@@ -386,7 +387,7 @@ class tst_QGeoMapScene : public QObject
             mapGeometry.setVisibleTiles(ct.tiles());
 
             QDoubleVector2D mercatorPos(mercatorX, mercatorY);
-            QPointF point = mapGeometry.mercatorToScreenPosition(mercatorPos);
+            QPointF point = mapGeometry.mercatorToScreenPosition(mercatorPos).toPointF();
 
             QCOMPARE(point.x(), screenX);
             QCOMPARE(point.y(), screenY);
